<?xml version="1.0" encoding="US-ASCII"?>
<?xml-stylesheet type="text/xsl" href="../../xslstyle/xslstyle-docbook.xsl"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
  xmlns:cs="http://www.CraneSoftwrights.com/ns/Crane-ods"
  xmlns:csint="internal-namespace-do-not-use-outside-of-this-module"
  xmlns:table="urn:oasis:names:tc:opendocument:xmlns:table:1.0"
  xmlns:text="urn:oasis:names:tc:opendocument:xmlns:text:1.0"
  xmlns:xs="http://www.CraneSoftwrights.com/ns/xslstyle"
  xmlns:xsd="http://www.w3.org/2001/XMLSchema"
  exclude-result-prefixes="cs table text xs xsd"
                version="2.0">

<xs:doc info="$Id: Crane-ods.xsl,v 1.7 2008/03/06 20:09:56 G. Ken Holman Exp $"
        filename="Crane-ods.xsl" global-ns="xs cs" 
        internal-ns="csint" vocabulary="DocBook">
  <xs:title>Crane's access to ODS XML instances</xs:title>
  <para>
    This provides XSLT 2.0 access to an ODS spreadsheet table.  The table is
    expected in an ODS XML instance, not a raw ODS ZIP package.  The 
    "content.xml" component of the ODS file ZIP package is the XML instance.
  </para>
</xs:doc>

<!--
     $Id: Crane-ods.xsl,v 1.7 2008/03/06 20:09:56 G. Ken Holman Exp $

  Copyright (C) - Crane Softwrights Ltd.
 		- http://www.CraneSoftwrights.com/links/res-dev.htm

  Redistribution and use in source and binary forms, with or without
  modification, are permitted provided that the following conditions are met:
  
  - Redistributions of source code must retain the above copyright notice,
    this list of conditions and the following disclaimer.
  - Redistributions in binary form must reproduce the above copyright notice,
    this list of conditions and the following disclaimer in the documentation
    and/or other materials provided with the distribution.
  - The name of the author may not be used to endorse or promote products
    derived from this software without specific prior written permission.
  
  THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
  IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES
  OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN
  NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED
  TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
  PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
  LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEG-
  LIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
  EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
  
  Note: for reference, the above is the "Modified BSD license", this text
  was obtained 2003-07-26 at http://www.xfree86.org/3.3.6/COPYRIGHT2.html#5
-->

<!--========================================================================-->
<xs:doc>
  <xs:title>Invocation parameter</xs:title>
</xs:doc>

<xs:param ignore-ns="yes">
  <para>The default document that contains the ODS XML.</para>
  <para>
    This can be overridden with an explicit document node to an ODS document.
  </para>
  <para>
    When this library is being indirectly included into a stylesheet, but the
    library isn't needed in any way, this parameter needs to be overridden so
    that it doesn't try to validate the source doucment as being an ODS file.
    The following variable declaration in an importing stylesheet will override
    the use of any file for this library:
  </para>
  <programlisting><![CDATA[<xsl:variable name="Crane-ods-document" select="/.." as="node()?"/>]]>
  </programlisting>
</xs:param>
<xsl:param name="Crane-ods-document" select="/" as="document-node()"/>

<!--========================================================================-->
<xs:doc>
  <xs:title>Entry points</xs:title>
</xs:doc>

<xs:function>
  <para>
    Having this avoids having to know about default table global variable.
  </para>
</xs:function>
<xsl:function name="cs:get-table" as="element(table:table)">
  <xsl:sequence select="$csint:table-check"/>
</xsl:function>

<xs:function>
  <para>
    Return the column number of the given node provided it is a table cell.
  </para>
  <xs:param name="cell">
    <para>The cell being examined.</para>
  </xs:param>
</xs:function>
<xsl:function name="cs:column-number" as="xsd:double+">
  <xsl:param name="cell" as="element(table:table-cell)"/>
  <xsl:for-each select="$cell">
    <xsl:variable name="base"
                  select="sum(preceding-sibling::table:*/
                                              @table:number-columns-repeated) -
                          count(preceding-sibling::table:*/
                                @table:number-columns-repeated) +
                          count(preceding-sibling::table:covered-table-cell) +
                          count(preceding-sibling::table:table-cell) + 1"/>
    <xsl:sequence select="for $each in $base cast as xsd:integer to ( $base + 
                              sum( @table:number-columns-repeated ) -
                              count( @table:number-columns-repeated ) )
                              cast as xsd:integer
                          return $each"/>
    <!--
    <xsl:message select="$base, sum( @table:number-columns-repeated ),
                         count( @table:number-columns-repeated ),
                         ':', for $each in $base cast as xsd:integer to ( $base + 
                              sum( @table:number-columns-repeated ) -
                              count( @table:number-columns-repeated ) )
                              cast as xsd:integer
                          return $each"/>
    -->
  </xsl:for-each>
</xsl:function>

<xs:function>
  <para>
    Obtain the set of table rows for the given table
  </para>
  <para>Note there is no distinction between rows in the header and rows in the table itself.</para>
  <xs:param name="table">
    <para>The table to access.</para>
  </xs:param>
</xs:function>
<xsl:function name="cs:get-rows" as="element(table:table-row)*">
  <xsl:param name="table" as="element(table:table)?"/>
  <xsl:sequence select="( $table, $csint:table-check )[1]/
                               ( table:table-row | 
                                 table:table-header-rows/table:table-row )"/>
</xsl:function>

<xs:function>
  <para>
    This returns the value of an addressed cell using three arguments: the
    table, the decimal row number and the decimal column number.
  </para>
  <para>
    Table cells are counted incorporating the number of columns spanned in
    previous columns
  </para>
  <xs:param name="table">
    <para>The table to access.</para>
  </xs:param>
  <xs:param name="col">
    <para>Numeric column to access.</para>
  </xs:param>
  <xs:param name="row">
    <para>Numeric row to access.</para>
  </xs:param>
</xs:function>
<xsl:function name="cs:get-cell" as="xsd:string">
  <xsl:param name="table" as="element(table:table)?"/>
  <xsl:param name="col" as="xsd:double"/>
  <xsl:param name="row" as="xsd:double"/>
  <xsl:value-of select="string-join( cs:get-rows($table)[$row]/
                        table:table-cell[cs:column-number(.) = $col]/text:p,
                        '' )"/>
</xsl:function>

<xs:function>
  <para>
    This returns the value of an addressed cell using two arguments.
  </para>
  <para>
    There are two ways to use this function, based on the type of the arguments
  </para>
  <xs:param name="ref">
    <para>
      When this is the empty sequence (implying the default table) or is a
      <literal>table:table</literal> then the second argument
      is expected to be the alphanumeric address of the cell to access 
      encoded as {A-Z}{n}, e.g. <literal>B3</literal>.
    </para>
    <para>
      When this is <literal>table:table-row</literal> then the second argument
      is expected to be a decimal column number.
    </para>
    <para>
      Other types of the first argument flag errors.
    </para>
  </xs:param>
  <xs:param name="col-row">
    <para>
      See the definition of <literal>$ref</literal> for the two possible
      values for this parameter.
    </para>
    <para>
      Note the implementation limitation at this time for alphanumeric cell 
      addresses: there can only be a single letter in the column 
      specification, multiple letters are not supported at this time.
    </para>
  </xs:param>
</xs:function>
<xsl:function name="cs:get-cell" as="xsd:string">
  <xsl:param name="ref" as="element()?"/>
  <xsl:param name="col-row" as="xsd:anyAtomicType?"/>
  <xsl:variable name="table" select="( $ref, $csint:table-check )[1]"/>
  <xsl:choose>
    <xsl:when test="$table instance of element(table:table)">
      <!--the second argument is an alphanumeric cell reference-->
      <xsl:choose>
        <xsl:when test="exists($col-row)">
          <!--the second argument is supplied-->
          <xsl:value-of
            select="cs:get-cell($table,
                                string-to-codepoints(substring($col-row,1,1))-
                                 string-to-codepoints('A')+1,
                                 number(substring($col-row,2)) )"/>
        </xsl:when>
        <xsl:otherwise>
          <!--no column/row reference was given, so return the empty string-->
          <xsl:text></xsl:text>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:when>
    <xsl:when test="$table instance of element(table:table-row)">
      <!--the second argument is required and must be a decimal cell number-->
      <xsl:choose>
        <xsl:when test="$col-row instance of xsd:decimal">
          <xsl:value-of select="string-join( $table/
                       table:table-cell[cs:column-number(.) = $col-row]/text:p,
                                             '' )"/>
        </xsl:when>
        <xsl:otherwise>
          <xsl:sequence 
     select="error(QName('http://www.w3.org/2005/xqt-errors', 'x:FOER0000'),
'The second argument to get-cell() is required when the first argument is a table row')"/>
        </xsl:otherwise>
      </xsl:choose>
    </xsl:when>
    <xsl:otherwise>
      <!--the first argument isn't what was expected-->
      <xsl:sequence 
     select="error(QName('http://www.w3.org/2005/xqt-errors', 'x:XPTY0004'),
                   concat('Unexpected element type: ',name($table)))"/>
    </xsl:otherwise>
  </xsl:choose>
</xsl:function>

<!--========================================================================-->
<xs:doc>
  <xs:title>Internal parameter</xs:title>
</xs:doc>

<xs:variable>
  <para>
    An internal-use-only variable that gracefully reports an unexpected
    document.  This expects lazy evaluation, so that if the variable is not
    accessed, then it is not evaluated.  A processor that does not do lazy
    evaluation may incorrectly report an error when the user never means to
    use this variable.
  </para>
</xs:variable>
<xsl:variable name="csint:table-check" as="element(table:table)?">
  <xsl:if test="$Crane-ods-document and 
                not( $Crane-ods-document//table:table[1] )">
    <xsl:message terminate="yes">
      <xsl:text>The supplied ODS document XML parameter does not </xsl:text>
      <xsl:text>contain an ODF table: </xsl:text>
      <xsl:value-of select="document-uri( $Crane-ods-document )"/>
      <xsl:text>
</xsl:text>
    </xsl:message>
  </xsl:if>
  <xsl:sequence select="($Crane-ods-document//table:table)[1]"/>
</xsl:variable>

</xsl:stylesheet>